<?php
/**
 * Fichier gérant les autorisations du plugin
 *
 * @plugin     erdv
 * @copyright  2024
 * @author     Vincent CALLIES
 * @licence    GNU/GPL
 * @package    SPIP\erdv\Installation
 */

if (!defined('_ECRIRE_INC_VERSION')) return;
 
// declaration vide pour ce pipeline.
function erdv_autoriser(){}
 
/**
 * Deux objets éditoriaux : 
 * - erdvs
 * - ecalendriers
 */
 
// -----------------
// Objet Rendez-vous (erdv)

// bouton de menu
function autoriser_erdvs_menu_dist($faire, $type, $id, $qui, $opts){
	 return TRUE;
}

// bouton de menu Outils rapides
function autoriser_erdvcreer_menu_dist($faire, $type, $id, $qui, $opt) {
	 return $qui['statut'] == '0minirezo' AND !$qui['restreint'];
}

// iconifier (mettre un logo)
function autoriser_erdv_iconifier_dist($faire, $type, $id, $qui, $opt) {
	// pas de logo sur un rendez-vous : il faut qu'un rendez-vous reste un objet simple
	return FALSE;
}

// creer
function autoriser_erdv_creer_dist($faire, $type, $id, $qui, $opt) {
	// donner l'autorisation de créer, sachant que le formulaire
	// prend le relai pour voir où
	return true;
}
 
// voir
function autoriser_erdv_voir_dist($faire, $type, $id, $qui, $opt) {
	// l'admin peut tout
	if ( $qui['statut'] == '0minirezo' AND !$qui['restreint']) return TRUE;
	// l'auteur associé au calendrier peut voir le rendez-vous
	if ( $id_auteur = (int) $qui['id_auteur']
		and $id = (int) $id
		and $type == 'erdv'
		and (
				( isset($opt['id_ecalendrier']) and $id_ecalendrier = (int) $opt['id_ecalendrier'] )
			or
				( $id_ecalendrier = sql_getfetsel('id_ecalendrier', 'spip_erdvs', 'id_erdv=' . $id) )
			)
		and include_spip('action/editer_liens')
		and objet_trouver_liens(['id_auteur' => $id_auteur], ['ecalendrier' => $id_ecalendrier])
	){
		return TRUE;
	}
	// si c'est une liste ($id = 0) il faut alors que l'$id_ecalendrier ait été précisé en option
	// et que l'auteur associé au calendrier indiqué pour voir les rendez-vous
	if (
		!$id = (int) $id
		and isset($opt['id_ecalendrier'])
		and $id_ecalendrier = (int) $opt['id_ecalendrier']
		and include_spip('action/editer_liens')
		and objet_trouver_liens(['id_auteur' => $id_auteur], ['ecalendrier' => $id_ecalendrier])
	){
		return TRUE;
	}
	return FALSE;
}
 
// modifier
function autoriser_erdv_modifier_dist($faire, $type, $id, $qui, $opt) {
	// l'admin peut tout
	if ( $qui['statut'] == '0minirezo' AND !$qui['restreint']) return TRUE;
	// l'auteur associé au calendrier peut modifier le rendez-vous
	if ( $id_auteur = (int) $qui['id_auteur']
		and $id = (int) $id
		and $type == 'erdv'
		and (
				( isset($opt['id_ecalendrier']) and $id_ecalendrier = (int) $opt['id_ecalendrier'] )
			or
				( $id_ecalendrier = sql_getfetsel('id_ecalendrier', 'spip_erdvs', 'id_erdv=' . $id) )
			)
		and include_spip('action/editer_liens')
		and objet_trouver_liens(['id_auteur' => $id_auteur], ['ecalendrier' => $id_ecalendrier])
	){
		return TRUE;
	}
	return FALSE;
}

// instituer (changer les statuts)
function autoriser_erdv_instituer_dist ($faire, $type, $id, $qui, $opt) {
	// l'admin peut tout
	if ($qui['statut'] == '0minirezo' and !$qui['restreint']){
		return true;
	}
	// les autres ne peuvent ni archiver ni anonymiser 
	return !in_array($opt['statut'], ['archive', 'anonymise']);
}

// -----------------
// Objet Calendrier (ecalendrier)

// créer un erdv dans un ecalendrier
function autoriser_ecalendrier_creererdvdans_dist($faire, $type, $id, $qui, $opt) {
	// l'admin peut tout
	if ( $qui['statut'] == '0minirezo' AND !$qui['restreint']) return TRUE;
	// l'auteur associé à un calendrier peut y créer des rendez-vous 
	if ($id = (int) $id){
		return autoriser('voir', 'erdv', 0, $qui, ['id_ecalendrier' => $id]);
	}
	return FALSE; 
}

// bouton de menu
function autoriser_ecalendriers_menu_dist($faire, $type, $id, $qui, $opts){
	 return $qui['statut'] == '0minirezo' AND !$qui['restreint'];
}

function autoriser_ecalendriersemaine_menu_dist($faire, $type, $id, $qui, $opts){
	 return TRUE;
 }
 
// creer
function autoriser_ecalendrier_creer_dist($faire, $type, $id, $qui, $opt) {
	 return $qui['statut'] == '0minirezo' AND !$qui['restreint']; 
}
 
// voir
function autoriser_ecalendrier_voir_dist($faire, $type, $id, $qui, $opt) {
	return autoriser('voir', 'erdv', '0', $qui, ['id_calendrier' => $id]);
}
 
// modifier
function autoriser_ecalendrier_modifier_dist($faire, $type, $id, $qui, $opt) {
	 return $qui['statut'] == '0minirezo' AND !$qui['restreint']; 
}
 
 // supprimer
function autoriser_ecalendrier_supprimer_dist($faire, $type, $id, $qui, $opt) {
	return $qui['statut'] == '0minirezo' AND !$qui['restreint']; 
}
 
// associer (lier / delier)
function autoriser_associerecalendrier_dist($faire, $type, $id, $qui, $opt) {
	 return $qui['statut'] == '0minirezo' AND !$qui['restreint']; 
}

 